/*
    DoubleSimple: DoubleKahanSum.h
    Copyright (C) 2003-2006  Macaulay Institute

    This file is part of DoubleSimple, a series of classes for implementing
    various remedies to floating point arithmetic errors.

    DoubleSimple is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    DoubleSimple is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details. (LICENCE file in
    this directory.)

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Contact information:
      Gary Polhill,
      Macaulay Institute, Craigiebuckler, Aberdeen, AB15 8QH. United Kingdom
      g.polhill@macaulay.ac.uk
*/
/*

This class over-rides the addition and subtraction methods to implement the
Kahan summation algorithm. This algorithm appears in the Sun Numerical
Computation Guide July 2001 on p. 214 in Appendix D, Theorem 8. Appendix D
contains an edited reproduction of a paper by David Goldberg in the March
1991 issue of Computing Surveys entitled "What every computer scientist
should know about floating-point arithmetic".

*/

#import <DoubleSimple.h>

@interface DoubleKahanSum: DoubleSimple {
  double correction;
}

/* Methods to override from DoubleSimple */

+newUp;

-set: (DoubleSimple *)value;
-cset: (double)value;
-setStr: (char *)str;

-read: (FILE *)fp;

-add: (DoubleSimple *)value;
-sub: (DoubleSimple *)value;

-iadd: (int)value;
-isub: (int)value;

-cadd: (double)value;
-csub: (double)value;

@end
