/*
    FEARLUS model0-7-1ci: Environment.h
    Copyright (C) 1999-2003, 2006  Macaulay Institute

    This file is part of FEARLUS model0-7-1ci, an agent-based model of land use
    change.

    FEARLUS model0-7-1ci is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    FEARLUS model0-7-1ci is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details. (LICENCE file in
    this directory.)

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Contact information:
      Gary Polhill,
      Macaulay Institute, Craigiebuckler, Aberdeen, AB15 8QH. United Kingdom
      g.polhill@macaulay.ac.uk
*/
/*


Interface for the environment class. The environment is implemented as
a 2D grid, each of which contains a land parcel. The grid is
optionally toroidal, cylindrical or bounded. The environment also
contains the climate and economic factors, and a list of the valid
land use types.


*/

#import <space/Discrete2d.h>
#import <collections/List.h>
#import <random.h>
#import <errno.h>
#import <stdio.h>
#import <objc/objc-api.h>	// sel_get_name
#import <simtools.h>		// InFile and OutFile

#import "Topology.h"
#import "Neighbourhood.h"

@class BitString, Parameter, LandParcel, Coordinate2D, Interval;

typedef enum climate_economy_file_status {
  NOT_OPENED_YET, OPENED_FOR_READING, OPENED_FOR_WRITING, CLOSED
} file_status_t;

@interface Environment: Discrete2d {
  // Instance variables with a causal influence
  id <Grid2DTopology> topology;		// Planar, Toroidal, Cylindrical...
  id <Neighbourhood> neighbourhood;	// Moore, VonNeumann, Triangle, Hex...
  Parameter *parameter;
  BitString *climate;
  FILE *climateFile;
  file_status_t climateFileStatus;
  FILE *economyFile;
  file_status_t economyFileStatus;
  BitString *economy;
  id landUses;
  id landParcels;		// Need to keep a list of the land parcels to
				// pass back to the ObserverSwarm
  unsigned sClimate;		// Model parameter for size of climate string
  unsigned sEconomy;		// Model parameter for size of economy string
  unsigned sLandParcel;		// Model parameter for size of land parcel 
				// string
  unsigned sLandUse;		// The size of a land use string (= climate +
				// economy + land parcel)
  unsigned nLandUse;		// Model parameter for number of land use types
  Interval *pLUDontCare;	// Model parameter for probability of a don't
				// care bit in the LU type bit string
  unsigned nClumpCycles;	// Model parameter for number of clumping
				// cycles that take place in initial set up of
				// land parcels.
  // Instance variables for internal machinations
  Coordinate2D *n_centre;
				// used by the getNextNeighbour method
  // Instance variables for observation purposes
  unsigned year;
  int maxColour;
  id <List> history;
  char **nbrscreen;
  int nbrscreenmax;
  BOOL nbrprintedflag;
}

// No create method: Just use the superclass create method

-(void)setParameters: (Parameter *)p;

-(void)setMaxColour: (int)col;
-(void)calculateYield;
-(void)determineClimate;
-getClimate;
-(void)determineEconomy;
-getEconomy;
-getLandParcels;
-(void)createLandUses;
-(void)createLandParcels;
-(void)createLandParcelAtX: (int)x Y: (int)y;
-(LandParcel *)getLandParcelAtX: (int)x Y: (int)y;
-(void)clumpLandParcels;
-(void)printTextParcelBitstrings;
-(void)printTextParcelBitstringsBit: (unsigned)a;
-getLandUses;
-(unsigned)getNLandUse;
-connectLandParcels;
-(unsigned)getEconomyBitLength;
-(unsigned)getClimateBitLength;
-(unsigned)getLandParcelBitLength;
-(unsigned)getLandUseBitLength;
-(id <Grid2DTopology>)getTopology;
-(void)newYear;
-(unsigned)getYear;
-(void)rankLandParcels;
-(void)writeHistory: (FILE *)fp;
-saveLandParcelsToFileNamed: (char *)fname;
-saveLandUsesToFileNamed: (char *)fname;
-loadLandParcelsFromFileNamed: (char *)fname;
-loadLandUsesFromFileNamed: (char *)fname;
-(void)recordHistory;
-(void)drop;

@end
