/*
    FEARLUS model0-7-1ci: GlobalTopology.m
    Copyright (C) 1999-2003, 2006  Macaulay Institute

    This file is part of FEARLUS model0-7-1ci, an agent-based model of land use
    change.

    FEARLUS model0-7-1ci is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    FEARLUS model0-7-1ci is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details. (LICENCE file in
    this directory.)

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Contact information:
      Gary Polhill,
      Macaulay Institute, Craigiebuckler, Aberdeen, AB15 8QH. United Kingdom
      g.polhill@macaulay.ac.uk
*/


#import "GlobalTopology.h"
#import "GlobalNeighbourhood.h"
#import "Coordinate2D.h"
#import <stdio.h>
#import <objc/objc-api.h>

@implementation GlobalTopology

+create: aZone setSize: (id <Grid2DSpatial>)size 
              nbrClass: (Class)nbrh
                radius: (unsigned)r {
  GlobalTopology *obj;

  if(nbrh != [GlobalNeighbourhood class]) {
    fprintf(stderr, "WARNING: GlobalTopology used with neighbourhood class "
	    "%s that is not GlobalNeighbourhood. Unexpected neighbourhood "
	    "behaviour is possible.\n", class_get_class_name(nbrh));
  }
  obj = [super create: aZone setSize: size nbrClass: nbrh radius: r];

  return obj;
}

/*

globalX

Method to set search parameters for a global topology.

*/

-(void)globalX {
  minCoord->x = 0;
  maxCoord->x = sizeCoord->x - 1;
}

/*

globalY

Method to set search parameters for a global topology.

*/

-(void)globalY {
  minCoord->y = 0;
  maxCoord->y = sizeCoord->y - 1;
}

/*

startSearchWithCentre:

Set things up for a new search at the specified centre location.

*/

-(id <Grid2DSpatial>)startSearchWithCentre: (id <Grid2DSpatial>)loc {
  return [self startSearchWithCentre: loc
	       Xmethod: M(globalX)
	       Ymethod: M(globalY)];
}

@end
